-----------------------------------
    Population: LIP_0_0
-----------------------------------
  N=272
  C=0.500000 nF
  Membrane time constant=20.000000 ms
  Resting potential=-70.000000 mV
  Reset potential=-55.000000 mV
  Threshold =-50.000000 mV
  RestPot_ca =-85.000000 mV
  Alpha_ca =0.500000 mV
  Tau_ca =80.000000 mV
  Eff_ca =0.000000 mV
  tauhm=20.000000 ms
  tauhp=100.000000 ms
  V_h=-60.000000 mV
  V_T=120.000000 mV
  g_T=0.000000 mS
Receptor 0: GABA (Mg block: 0)
  Tau=5.000000 ms
  Reversal potential=-70.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Receptor 1: AMPA (Mg block: 0)
  Tau=2.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=2.200000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=2.000000 nS
  Ext connections=800.000000
Receptor 2: NMDA (Mg block: 1)
  Tau=100.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Synapses targeting population: D1STR_0_0 (8)
   Target receptor code=1
  Connectivity=1.000000
  Mean efficacy=0.229500
Synapses targeting population: D1STR_0_0 (8)
   Target receptor code=2
  Connectivity=1.000000
  Mean efficacy=0.229500
Synapses targeting population: D2STR_0_0 (10)
   Target receptor code=1
  Connectivity=1.000000
  Mean efficacy=0.225000
Synapses targeting population: D2STR_0_0 (10)
   Target receptor code=2
  Connectivity=1.000000
  Mean efficacy=0.225000
Synapses targeting population: FSI_0 (0)
   Target receptor code=1
  Connectivity=1.000000
  Mean efficacy=0.180000
Synapses targeting population: Th_0_0 (14)
   Target receptor code=1
  Connectivity=0.875000
  Mean efficacy=0.033500
Synapses targeting population: Th_1_0 (15)
   Target receptor code=1
  Connectivity=0.875000
  Mean efficacy=0.033500
Synapses targeting population: Th_0_0 (14)
   Target receptor code=2
  Connectivity=0.875000
  Mean efficacy=0.033500
Synapses targeting population: Th_1_0 (15)
   Target receptor code=2
  Connectivity=0.875000
  Mean efficacy=0.033500
Synapses targeting population: LIP_0_0 (12)
   Target receptor code=1
  Connectivity=0.325000
  Mean efficacy=0.012700
Synapses targeting population: LIP_1_0 (13)
   Target receptor code=1
  Connectivity=0.325000
  Mean efficacy=0.012700
Synapses targeting population: LIP_0_0 (12)
   Target receptor code=2
  Connectivity=0.325000
  Mean efficacy=0.150000
Synapses targeting population: LIP_1_0 (13)
   Target receptor code=2
  Connectivity=0.325000
  Mean efficacy=0.150000
Synapses targeting population: LIPI_0 (1)
   Target receptor code=1
  Connectivity=0.181250
  Mean efficacy=0.013000
Synapses targeting population: LIPI_0 (1)
   Target receptor code=2
  Connectivity=0.181250
  Mean efficacy=0.125000
EndPopulation



import os
import numpy as np
import pandas as pd
from bgNetwork.analyzefx import *
from scipy.stats.distributions import norm
from scipy.stats import truncnorm
import matplotlib.pyplot as plt
import seaborn as sns
import warnings
warnings.simplefilter('ignore', np.RankWarning)

roundRT = lambda x, ndec: int(round(float(str(x)[:5]), ndec)*1000)



def plot_behavior(empdata, simdata, labels=['low', 'med', 'high'], simclrs=['#2876B9'], simlabels=['Drift']):

    f, (ax1, ax2) = plt.subplots(1, 2, figsize=(8.5,4.5))
    x = np.arange(1, len(labels)+1)
    xtl = labels
    rtemp, rtempErr, accemp, accempErr = empdata
    ax1.errorbar(x, rtemp, yerr=rtempErr, marker='o', markersize=8, linewidth=2.5, elinewidth=2.5, color='k', mec='k', mew=2,label='CBGT Network')
    ax2.errorbar(x, accemp, yerr=accempErr, marker='o', markersize=8, linewidth=2.5, elinewidth=2.5, color='k', mec='k', mew=2)

    for i, simd in enumerate(simdata):
        rtsim, rtsimErr, accsim, accsimErr = simd
        c = simclrs[i]
        l = simlabels[i]
        ax1.errorbar(x+.04, rtsim, yerr=rtsimErr, mfc=c, marker='o', markersize=6,
                     linewidth=2., elinewidth=2.5, mec=c, mew=2, color=c, label=l)
        ax2.errorbar(x+.04, accsim, yerr=accsimErr, mfc=c, marker='o', markersize=6,
                     linewidth=2., elinewidth=2.5, mec=c, mew=2, color=c)

    for ax in [ax1, ax2]:
        ax.set_xlim(x[0]-.2, x[-1]+.2)
        ax.set_xticks(x)
        ax.set_xticklabels(xtl)
    yt=np.arange(.20, .45, .05)
    ax1.set_yticks(yt)
    ax1.set_yticklabels([int(round(ytl,2)) for ytl in yt*1000])
    ax2.set_yticks(np.arange(.4, 1.1, .1))
    ax2.set_ylim(.4, 1.05)
    ax1.set_ylabel('RT (ms)')
    ax2.set_ylabel('Accuracy')
    ax1.set_xlabel('D/I Ratio (All Channels)')
    ax2.set_xlabel('D/I Ratio (All Channels)')
    ax1.legend()
    sns.despine()
    plt.tight_layout()


def plot_mean_acc_rt(df, conds=['low', 'med', 'high'], subject_mean=False, clrs=['#347fff', '#00bac7', '#febe08'], eclrs=['#1657de', '#009c82', '#f5a005']):

    pal = sns.color_palette(eclrs)
    f, (ax1, ax2) = plt.subplots(1, 2, figsize=(10,5))
    if subject_mean:
        dfacc = df.groupby(['subj_idx', 'level']).mean().reset_index()
        dfRT = df[df.acc==1].groupby(['subj_idx', 'level']).mean().reset_index()
    else:
        dfacc = df.copy()
        dfRT = df[df.acc==1].copy()

    x = np.arange(len(conds))

    yacc = dfacc.groupby('level').mean()['acc'].values
    yrt =  dfRT.groupby('level').mean()['rtms'].values
    yaccERR = dfacc.groupby('level').sem()['acc'].values*1.96
    yrtERR =  df[df.acc==1].groupby('level').sem()['rtms'].values*1.96

    ax1.plot(x, yacc, linewidth=4, alpha=.25, color='k')
    ax2.plot(x, yrt, linewidth=4, alpha=.25, color='k')

    for i in range(len(conds)):
        ax1.errorbar([i], [yacc[i]], yerr=[yaccERR[i]], color=clrs[i], marker='o', linewidth=0, elinewidth=3, mfc=clrs[i], mew=2, mec=eclrs[i], ms=12)
        ax2.errorbar([i], [yrt[i]], yerr=[yrtERR[i]], color=clrs[i], marker='o', linewidth=0, elinewidth=3, mfc=clrs[i], mew=2, mec=eclrs[i], ms=12)

    for ax in [ax1, ax2]:
        ax.set_xticks(range(len(conds)))
        ax.set_xticklabels([c.capitalize() for c in conds])
        ax.set_xlabel('P(rew | Left)')
        ax.set_xlim(x[0]-.25, x[1]+.25)
        sns.despine(ax=ax)

    ax1.set_ylim(.4, 1.025)
    ax1.set_yticks([.4, .6, .8, 1.])
    ax1.set_yticklabels([.4, .6, .8, 1.])
    ax1.set_ylabel('Accuracy')
    ax2.set_ylabel('RT (ms)')
    plt.tight_layout()


def plot_cond_rtdist(bdf, bins=12, outdir=None, cond='low', norm_hist=False):
    #splot.style(fscale=1.3)
    f, (ax1, ax2) = plt.subplots(1, 2, figsize=(8,3), sharey=True)
    sns.distplot(bdf[bdf.acc==1].rtms.values, color='k', ax=ax1, kde=False,
                 norm_hist=norm_hist, label='Left', bins=bins)
    if bdf[bdf.acc==0].shape[0] > 2:
        sns.distplot(bdf[bdf.acc==0].rtms.values, color='r', ax=ax2, kde=False,
                     norm_hist=norm_hist, label='Right', bins=int(bins/2))
        ax2.legend()
    f.suptitle(cond.capitalize())
    sns.despine(); plt.tight_layout
    ax1.set_ylabel('Probability Density')
    ax1.set_xlabel('RT (ms)')
    ax2.set_xlabel('RT (ms)')
    ax1.legend()
    for i, ax in enumerate([ax2, ax1]):
        ax.set_yticklabels([])
        xmin, xmax = [roundRT(xx, 2) for xx in [bdf.rt.min(), bdf.rt.max()]]
        ax.set_xticks(np.arange(xmin, xmax+100, 100))
        ax.set_xticklabels(np.arange(xmin, xmax+100, 100))
        ax.set_xlim(xmin-20, xmax)
    plt.tight_layout()
    if outdir is not None:
        plt.savefig(os.path.join(outdir, '{}_rt_dists.png'.format(cond)), dpi=400)


def plot_cor_err_rts(bdf, bins=12, outdir=None, norm_hist=False):
    f, ax = plt.subplots(1, figsize=(6.5,3.9))
    cond = bdf.cond.iloc[0]
    sns.distplot(bdf[bdf.acc==1].rtms.values, label='Left', color='Green', bins=bins, kde=False, ax=ax, norm_hist=norm_hist)
    ax = sns.distplot(bdf[bdf.acc==0].rtms.values, label='Right', color='Red', bins=int(bins/2), kde=False, ax=ax, norm_hist=norm_hist)
    ax.legend(); ax.set_title(cond.capitalize()); sns.despine()
    xmin=roundRT(bdf.rt.min(), 2); xmax=roundRT(bdf.rt.max(), 2)
    ax.set_xlim(xmin-25, xmax+50)
    ax.set_yticklabels([])
    ax.set_ylabel('Probability Density')
    ax.set_xlabel('RT (ms)')
    plt.tight_layout()
    if outdir is not None:
        plt.savefig(os.path.join(outdir, '{}_cor_err_rt_dists.png'.format(cond)), dpi=400)


def plot_stim_dist(mu=2.49, sd=.025, clip=None, ntrials=1000, bins=25):
    if clip is not None:
        lower, upper = clip[0], clip[1]
        a = (lower - mu) / sd
        b = (upper - mu) / sd
        stimArray = truncnorm(a, b, loc=mu, scale=sd).rvs(ntrials)
    else:
        stimArray = norm(mu, sd).rvs(ntrials)
    _=plt.hist(stimArray, bins=bins); sns.despine()
    return stimArray


def plot_average_msn_rates(ys, ysErr, bdf, ntime=500,  savedir=None, conds=['low', 'med', 'high'], clrs=['#347fff', '#00bac7', '#febe08'], eclrs=['#1657de', '#009c82', '#f5a005'], ymax=50, lw=2, plotRT=True, plotRTerr=False, outdir=None):

    nconds = len(conds)
    f, axes = plt.subplots(2, nconds, figsize=(nconds*3+2, 6), sharey=True, sharex=True)
    axes = np.asarray(axes).reshape(2,nconds)

    titles = [c.capitalize() for c in conds]
    lrclrs=['#1e1e1e', '#f5191c']
    lrEclrs = ['#b0b0b0', '#f9aeb0']

    pal = sns.color_palette(clrs)

    #condRTs = bdf.groupby('cond2').mean()['rtms'].values.astype(int)
    dfRT = bdf[bdf.acc==1].groupby(['subj_idx', 'level']).mean().reset_index()
    condRTs =  dfRT.groupby('level').mean()['rtms'].values
    rtErrs = dfRT.groupby('level').std()['rtms'].values

    for i in range(nconds):
        lbl0, lbl1 = None, None
        rt = condRTs[i]
        xmax = ntime
        if i==0:
            lbl0, lbl1 = 'Left', 'Right'
        y0, y1, y2, y3 = [y[i, :xmax] for y in ys]
        e0, e1, e2, e3 = [e[i, :y0.size] for e in ysErr]
        x = np.arange(200, 200+y0.shape[0])

        axes[0, i].plot(x, y0, linewidth=lw, color=lrclrs[0], label=lbl0, zorder=10)
        axes[0, i].plot(x, y1, linewidth=lw, color=lrclrs[1], label=lbl1, zorder=10)
        axes[0, i].fill_between(x, y0-e0, y0+e0, color=lrEclrs[0], alpha=.5, zorder=5)
        axes[0, i].fill_between(x, y1-e1, y1+e1, color=lrEclrs[1], alpha=.5, zorder=5)

        axes[1, i].plot(x, y2, linewidth=lw, color=lrclrs[0], zorder=10)
        axes[1, i].plot(x, y3, linewidth=lw, color=lrclrs[1], zorder=10)
        axes[1, i].fill_between(x, y2-e2, y2+e2, color=lrEclrs[0], alpha=.5, zorder=5)
        axes[1, i].fill_between(x, y3-e3, y3+e3, color=lrEclrs[1], alpha=.5, zorder=5)


        if plotRT:
            rtLower = rt-rtErrs[i]
            rtUpper = rt+rtErrs[i]

            axes[0, i].vlines(rt, 0, ymax, linewidth=2.5, color=clrs[i], linestyle='-', alpha=.85, zorder=20)
            axes[1, i].vlines(rt, 0, ymax, linewidth=2.5, color=clrs[i], linestyle='-', alpha=.85, zorder=20)
            axes[0, i].plot([rt], [ymax], linewidth=0, marker='o', ms=8, color=clrs[i], zorder=15)
            axes[1, i].plot([rt], [ymax], linewidth=0, marker='o', ms=8, color=clrs[i], zorder=15)

            if plotRTerr:
                axes[0, i].fill_betweenx([0,ymax], [rtLower]*2,[rtUpper]*2, color=eclrs[i], alpha=.075, zorder=-10)
                axes[1, i].fill_betweenx([0,ymax], [rtLower]*2,[rtUpper]*2, color=eclrs[i], alpha=.075, zorder=-10)


    for ax in axes.flatten():
        xticks = np.linspace(205, ntime+5, 5)
        xtlabels = [str(int(xt-5)) for xt in xticks]
        ax.set_xticks(xticks)
        ax.set_xlim(202, ntime+8)
        ax.set_ylim(1., ymax+1)
        ax.set_yticks([1.5, 15, 30, 45])
        ax.set_yticklabels([0, 15, 30, 45])
        # ax.set_xticks(np.linspace(210, ntime+10, 5))
        # if ax.is_first_row():
        #     ax.set_xticklabels([])
        # else:
        #     #ax.set_xticks(np.linspace(210, ntime+10, 5))
        #     ax.set_xticklabels(['200', '', '', '', str(ntime)])
    ax.set_xticklabels(xtlabels)
    _=axes[0,0].legend(loc=2)
    _=axes[0,0].set_ylabel('dMSN Rate (Hz)')
    _=axes[1,0].set_ylabel('iMSN Rate (Hz)')
    for i, ax in enumerate(axes[0, :]):
        ax.set_title(titles[i])
    for i, ax in enumerate(axes[1, :]):
        ax.set_xlabel('Time (ms)')
    sns.despine()
    plt.tight_layout()

    if outdir is not None:
        plt.savefig(os.path.join(outdir, 'DI_AvgRates_muRT_15IDX_5TRIALS.png'), dpi=600)


def plot_spikes(df, start=200, nchoices=2, clrs=['#1e1e1e', '#f5191c']):
    f, axes = plt.subplots(2, 4, figsize=(12,4.75), sharex=True)
    axes = axes.flatten()
    if isinstance(df, dict):
        df = df['popfreqs'].values
    pops = ['LIP', 'dMSN', 'iMSN', 'FSI', 'GPe', 'STN', 'GPi', 'Th']
    LIPb_ix = 1
    M1_ix = 2
    x = df.loc[start:, 'time'].values
    ymax=[]
    for i in range(nchoices):
        choicelabel = ['Left', 'Right'][i]
        for ii, pop in enumerate(pops):
            ax = axes[ii]
            label = pop.split('_')[0]
            if pop in ['FSI', 'LIPI']:
                y = df.loc[start:, pop].values
                ax.plot(x, y, color='k', linewidth=2.25)
                ax.set_title(label)
                if i == 0:
                    ymax.append([])
                else:
                    ymax[ii].append(np.nanmax(y))
                continue
            else:
                y = df.loc[start:, '{}{}'.format(pop, i)].values
                clr = clrs[i]
            if i == 0:
                ymax.append([])
            ax.plot(x, y, color=clr, label=choicelabel, linewidth=2.25)
            ax.set_title(label)
            ymax[ii].append(np.nanmax(y))
            if axes[ii].is_last_row():
                axes[ii].set_xlabel('Time (ms)')
    upper = [np.nanmax(ym)*1.08 for ym in ymax]
    for i, ymax in enumerate(upper):
        axes[i].set_ylim(0, ymax)
        if pops[i] == 'LIP':
            axes[i].legend(loc=2)
        if axes[i].is_first_col():
            axes[i].set_ylabel('Firing Rate (Hz)')
    plt.tight_layout()
    sns.despine()
    return f



def plot_trial_rates(t=0, cond='test', window=None):
    pops = ['LIP0', 'LIPI', 'dMSN0', 'iMSN0', 'FSI',
            'GPeP0', 'GPi0', 'Th0']
    results = np.hstack(ng.readAllTrialResults(1,0,1))
    plotspikes = print_trial_acc_rt(results, t=t)
    if plotspikes:
        ratedf = results[t]['popfreqs']
        frdf2 = get_firing_rates(results, window=window, cond=cond)
        ratedfT = get_single_trial_ratedf(frdf2, trial=t)
        titles=['Cortex', 'dMSN', 'iMSN', 'FSI', 'GPe',  'STN', 'GPi', 'Thalamus']
        yranges = [(0,25), (0,50), (0,50), (0,30), (0, 125), (0,70), (0, 125), (0, 30)]
        xmax = np.around(ratedf.index.values[-1], 1)
        xticks = np.arange(200, xmax+100, 100)
        fig = plot_spikes(ratedfT, start=200)
        axes = np.asarray(fig.axes).flatten()
        for i, ax in enumerate(axes):
            title = titles[i]
            ax.set_title(title)
            ax.set_xticks(xticks)
            if ax.is_last_row():
                ax.set_xticks(xticks)
                ax.set_xticklabels(xticks)
        plt.tight_layout()


def plot_striatal_regressors(df, regressors=None, lbls=None, msn_type='d', subject_mean=False, savedir=None, conds=['low', 'med', 'high'], clrs=['#347fff', '#00bac7', '#febe08'], eclrs=['#1657de', '#009c82', '#f5a005']):

    #clrs = ['#347fff', '#00bac7', '#febe08']
    #eclrs = ['#1657de', '#009c82', '#f5a005']
    #eclrsx = ['#143773', '#00746e', '#d78214']

    if regressors is None:
        if msn_type=='i':
            regressors = ['nI_lrdiff_sum', 'nI_LR_mean']
            lbls = ['$I_L - I_R$', '$I_{all}$']
        else:
            regressors = ['nD_lrdiff_sum', 'nDI_ldiff_sum']
            lbls = ['$D_L - D_R$', '$D_L - I_L$']
    elif lbls is None:
        lbls=[None, None]
    f, ax = plt.subplots(1, figsize=(6,4))

    if subject_mean:
        y_idx = df.groupby(['level', 'subj_idx']).mean().reset_index().groupby('level')
        y = y_idx.mean()[regressors].values
        yerr = y_idx.sem()[regressors].values * 3.5
    else:
        y = df.groupby('level').mean()[regressors].values
        yerr = df.groupby('level').sem()[regressors].values * 3.5

    y1,y2 = y.T
    yerr1, yerr2 = yerr.T
    x1 = np.arange(0,len(conds))*2-.25
    x2 = np.arange(0,len(conds))*2+.25
    #[-.25, 1.75, 3.75]
    #[.25, 2.25, 4.25],
    plt.bar(x1, y1, width=.5, align='center', color=clrs, yerr=yerr1, label=lbls[0], error_kw={'elinewidth':2, 'ecolor':eclrs}, alpha=.65)
    plt.bar(x2, y2, width=.5, align='center', color=eclrs, yerr=yerr2, label=lbls[1], error_kw={'elinewidth':2, 'ecolor':eclrs}, alpha=.85)
    ax = plt.gca(); ax.set_xticks(x1+.25)
    ax.set_xticklabels([c.capitalize() for c in conds])
    ax.set_ylabel('$\sum_{t=0}^{RT} X(t)$')
    ax.set_xlabel('P(rew | Left)')
    ax.set_ylim(0.,1.);
    ax.set_yticks(np.arange(0.,1.2,.2))

    if lbls[0]:
        ax.legend(loc=2)
    sns.despine()
    plt.tight_layout()
    if savedir is not None:
        msn_reg_plot = os.path.join(savedir, '{}MSN_regressors_subjMeans.png'.format(msn_type))
        plt.savefig(msn_reg_plot, dpi=600)


def save_and_plot(results, behavdf, cond='bal', idx=1, savedir='~/cbgt', window=2, Start=200, trials=[None], getdfs=False, plot_avg=False):

    spikedf = get_firing_rates(results, window=window, cond=cond, idx=idx)
    avgRates = get_mean_firing_rates(spikedf)

    if trials[0] is not None:
        for t in trials:
            dftrial = get_single_trial_ratedf(spikedf, t)
            ftrial = plot_spikes(dftrial, start=Start)
            fname = 'rates_{}_trial{}.png'.format(cond, str(t))
            ftrial.savefig(os.path.join(savedir, fname), dpi=500)
        plt.close('all')

    if plot_avg:
        favg = plot_spikes(avgRates, start=Start)
        fname = 'rates_{}_avg.png'.format(cond)
        favg.savefig(os.path.join(savedir, fname), dpi=500)

    spikedfName = 'cbgt_spikedf_alltrials_{}_idx{}.csv'.format(cond, idx)
    avgRatesName = 'cbgt_spikedf_mean_{}_idx{}.csv'.format(cond, idx)
    #spikedf.to_csv(os.path.join(savedir, spikedfName), index=False)
    avgRates.to_csv(os.path.join(savedir, avgRatesName), index=False)
    if getdfs:
        return spikedf, avgRates



import os
import numpy as np
import pandas as pd
import bgNetwork.netgen as ng
import random
import warnings

warnings.simplefilter('ignore', np.RankWarning)

def run_cbgt_sweeps(stimArray, preset=None, rampingCTX=True, savedir=None,
                    cond='bal', ntrials=1000, Start=200,
                    popscale=0.5, BaseStim=0.0, Dynamic=30.0, Choices=2,
                    **kwargs):

    if savedir is None:
        savedir = os.path.join(os.path.expanduser('~'), 'cbgt')

    if not os.path.isdir(savedir):
        os.mkdir(savedir)
    ng.setDirectory(savedir)

    for t, stim in enumerate(stimArray):
        np.random.seed(seed=t)
        sweepcount = ng.configureSweep(t, experiment='mc',
        rampingCTX=rampingCTX, preset=preset, Start=Start, popscale=popscale,
        BaseStim=BaseStim, WrongStim=stim, RightStim=stim,
        Dynamic=Dynamic, Choices=Choices, **kwargs)

    ng.compileAndRunSweep(1, 0, ntrials)
    results = np.hstack(ng.readAllTrialResults(1,0,ntrials))
    return results


# def run_cbgt_sweeps_OLD(stimArray, preset=None, rampingCTX=False, savedir=None,
#                     cond='bal', ntrials=1000, Start=200,
#                     popscale=0.5, BaseStim=0.0, Dynamic=48.0, Choices=2,
#                     CxSTR=1.0, GPiExtEff=6.35, STNExtEff=1.65,
#                     STNExtFreq=4.6, CxFSI=0.18, CxTh=0.2, ThSTR=0.18, ThCx=0.05):
#
#     if savedir is None:
#         savedir = os.path.join(os.path.expanduser('~'), 'cbgt')
#     ng.setDirectory(savedir); t=0
#
#     for stim in stimArray:
#         sweepcount = ng.configureSweep(t, experiment='mc',
#         rampingCTX=rampingCTX, preset=preset, Start=Start, popscale=popscale,
#         BaseStim=BaseStim, WrongStim=stim, RightStim=stim,
#         Dynamic=Dynamic, Choices=Choices, CxSTR=CxSTR, GPiExtEff=GPiExtEff,
#         SNExtEff=STNExtEff, STNExtFreq=STNExtFreq, CxFSI=CxFSI, CxTh=CxTh,
#         ThSTR=ThSTR, ThCx=ThCx)
#         t += 1
#
#     ng.compileAndRunSweep(1, 0, ntrials)
#     results = np.hstack(ng.readAllTrialResults(1,0,ntrials))
#     return results


def single_trial_sim(t=0, stim=2.52, preset_dict=None):

    Choices     =    2
    BaseStim    =    [0.00]
    Dynamic     =    [20.0]
    Start       =    200
    popscale    =    0.3

    GPiExtEff   =   5.925
    STNExtEff   =   1.59
    STNExtFreq  =   4.45

    CxSTR      =    0.25
    CxTh       =    0.015
    CxFSI      =    0.2
    ThSTR      =    0.2
    ThCx       =    0.01


    sweepcount = ng.configureSweep(0, experiment='mc', preset=preset_dict,
                                    Start=Start, popscale=popscale,
                                    BaseStim=BaseStim,
                                    WrongStim=stim, RightStim=stim,
                                    Dynamic=Dynamic,  Choices=Choices,
                                    CxSTR=CxSTR, GPiExtEff=GPiExtEff,
                                    STNExtEff=STNExtEff, STNExtFreq=STNExtFreq,
                                    CxFSI=CxFSI, CxTh=CxTh, ThSTR=ThSTR,
                                    ThCx=ThCx, rampingCTX=True)
    ng.compileAndRunSweepALL(1, 0, 1)



#!usr/bin/env python
import os, sys
import random
from subprocess import call
from subprocess import Popen
from copy import deepcopy
import pickle
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt



# package_dir
_package_dir = os.path.dirname(os.path.realpath(__file__))


def set_post_learning_weights(dMSN=[1.01, 0.99], iMSN=[1.0, 1.0]):

    preset = [{'cxd': {'dest': [0, 1], 'mult': dMSN, 'src': [0, 1]},
              'cxi': {'dest': [0, 1], 'mult': iMSN, 'src': [0, 1]}}]
    return preset

def makeChannel(dim='brain', pops=[], subchannels=[]):
    return {'dim': dim,
            'subchannels': subchannels,
            'pops': pops}


def makePop(name, receptors=[], data={}, data_overrides={}):
    recept_dict = {}
    for receptor in receptors:
        r_overrides = {}
        if isinstance(receptor, list):
            r_overrides = {
                'MeanExtCon': receptor[1],
                'MeanExtEff': receptor[2],
                'FreqExt': receptor[3]
            }
            receptor = receptor[0]
        recept_dict[receptor['name']] = receptor['data'].copy()
        recept_dict[receptor['name']].update(r_overrides)
    data = data.copy()
    data.update(data_overrides)
    return {'name': name, 'data': data, 'receptors': recept_dict}


def makeReceptor(name, preset={}, preset_overrides={}):
    data = {'Tau': 0, 'RevPot': 0, 'FreqExt': 0,
            'FreqExtSD': 0, 'MeanExtEff': 0, 'MeanExtCon': 0}
    data.update(preset)
    data.update(preset_overrides)
    return {'name': name, 'data': data}


def makePath(src, targ, receptor, preset=['all'], connectivity=1, efficacy=1,
             STFT=0, STFP=0, STDT=0, STDP=0, name='', cmtype='eff', conmatrix=[]):
    return {'src': src,
            'targ': targ,
            'name': name,
            'receptor': receptor,
            'connectivity': connectivity,
            'efficacy': efficacy,
            'STFT': STFT,
            'STFP': STFP,
            'STDT': STDT,
            'STDP': STDP,
            'preset': preset,
            'cmtype': cmtype,
            'conmatrix': conmatrix}


def camP(connections, src, targ, receptor, preset=['all'], connectivity=1,
         efficacy=1, STFT=0, STFP=0, STDT=0, STDP=0, name='', cmtype='eff', conmatrix=[]):
    maxlen = 1
    if isinstance(receptor, list):
        maxlen = len(receptor)
    if isinstance(connectivity, list):
        maxlen = len(connectivity)
    if isinstance(efficacy, list):
        maxlen = len(efficacy)
    if isinstance(preset[0], list):
        maxlen = len(preset)

    if not isinstance(receptor, list):
        receptor = [receptor] * maxlen
    if not isinstance(connectivity, list):
        connectivity = [connectivity] * maxlen
    if not isinstance(efficacy, list):
        efficacy = [efficacy] * maxlen
    if not isinstance(preset[0], list):
        preset = [preset] * maxlen

    for rec, con, eff, pre in zip(receptor, connectivity, efficacy, preset):
        connections.append(makePath(src, targ, rec, pre,
                                    con, eff, STFT, STFP, STDT, STDP, name, cmtype, conmatrix))


def makeHandle(name, targ, path, receptor=None, con=0,
               eff=0, preset=['syn'], conmatrix=[]):
    return {'name': name,
            'targ': targ,
            'path': path,
            'receptor': receptor,
            'connectivity': con,
            'efficacy': eff,
            'STFT': 0,
            'STFP': 0,
            'STDT': 0,
            'STDP': 0,
            'preset': preset,
            'cmtype': 'con',
            'conmatrix': conmatrix}


def makeHandleEvent(label, time, hname, hpath, freq, etype='ChangeExtFreq'):
    return {'label': label,
            'time': time,
            'hname': hname,
            'hpath': hpath,
            'freq': freq,
            'etype': etype}


def makeEvent(time, etype, label='', pop='', freq=0, receptor=''):
    return {'time': time,
            'etype': etype,
            'label': label,
            'pop': pop,
            'receptor': receptor,
            'freq': freq}


def constructCopies(dims, path, index=0):
    if index >= len(path):
        return [[]]
    copylist = []
    dimname = path[index]
    for i in range(0, dims[dimname]):
        subcopylist = constructCopies(dims, path, index + 1)
        for copy in subcopylist:
            copy.insert(0, i)
            copylist.append(copy)
    return copylist


def constructPopPaths(channel):
    poppaths = {}
    dimname = channel['dim']
    for pop in channel['pops']:
        poppaths[pop['name']] = [dimname]
    for subchannel in channel['subchannels']:
        subpoppaths = constructPopPaths(subchannel)
        for name, path in subpoppaths.items():
            subpoppaths[name].append(dimname)
        poppaths.update(subpoppaths)
    return poppaths


def constructPopCopies(dims, channel, poppaths):
    poplist = []
    for pop in channel['pops']:
        pop['path'] = poppaths[pop['name']]
        pathvals = constructCopies(dims, pop['path'])
        for pathval in pathvals:
            popcopy = pop.copy()
            popcopy['data'] = pop['data'].copy()
            popcopy['pathvals'] = pathval
            popcopy['uniquename'] = popcopy['name']
            for val in pathval:
                popcopy['uniquename'] += '_' + str(val)
            popcopy['targets'] = []
            poplist.append(popcopy)
    for subchannel in channel['subchannels']:
        subpoplist = constructPopCopies(dims, subchannel, poppaths)
        for pop in subpoplist:
            poplist.append(pop)
    return poplist


def constructConMatrix(dims, connection, path1, path2):
    # supports 'sym' 'all' 'anti' 'randbool'
    dim1 = path1[0]
    dim2 = path2[0]
    adj = []
    preset = connection['preset']
    for dist1 in range(0, dims[dim1]):
        adjrow = []
        for dist2 in range(0, dims[dim2]):
            con = 0
            if preset[0] == 'syn':
                if dist1 == dist2:
                    con = 1
            if preset[0] == 'anti':
                if dist1 != dist2:
                    con = 1
            if preset[0] == 'all':
                con = 1
            if preset[0] == 'randbool':
                if random.uniform(0, 1) < preset[1]:
                    con = 1
            adjrow.append(con)
        adj.append(adjrow)
    connection['conmatrix'] = adj


def constructTracts(connection, source, popcopylist):
    for target in popcopylist:
        if target['name'] == connection['targ']:
            i1 = len(source['pathvals']) - 1
            i2 = len(target['pathvals']) - 1
            valid = True
            while i1 >= 0 and i2 >= 0 and (i1 > 0 or i2 > 0):
                if source['path'][i1] != target['path'][i2]:
                    break
                if source['pathvals'][i1] != target['pathvals'][i2]:
                    valid = False
                    break
                i1 -= 1
                i2 -= 1
            if valid:
                dist1 = source['pathvals'][0]
                dist2 = target['pathvals'][0]
                conmod = connection['conmatrix'][dist1][dist2]
                if conmod > 0:
                    tract = {}
                    data = {}
                    tract['target'] = target['uniquename']
                    tract['name'] = connection['name']
                    data['TargetReceptor'] = connection['receptor']
                    data['STFacilitationTau'] = connection['STFT']
                    data['STFacilitationP'] = connection['STFP']
                    data['STDepressionTau'] = connection['STDT']
                    data['STDepressionP'] = connection['STDP']
                    data['Connectivity'] = connection['connectivity']
                    if connection['cmtype'] == 'con':
                        data['Connectivity'] *= conmod
                    data['MeanEff'] = connection['efficacy']
                    if connection['cmtype'] == 'eff':
                        data['MeanEff'] *= conmod
                    tract['data'] = data
                    source['targets'].append(tract)


def constructConnections(dims, connections, poppaths, popcopylist, **kwargs):
    for con in connections:
        constructConMatrix(
            dims, con, poppaths[con['src']], poppaths[con['targ']])
        for key, value in kwargs.items():
            if key == con['name']:
                if isinstance(value, dict):
                    for src,dest,mult in zip(value['src'],value['dest'],value['mult']):
                        con['conmatrix'][src][dest] *= mult
        for source in popcopylist:
            if source['name'] == con['src']:
                constructTracts(con, source, popcopylist)


def constructHandleCopies(dims, handletypes, poppaths, popcopylist):
    handles = []
    for handle in handletypes:
        constructConMatrix(
            dims, handle, handle['path'], poppaths[handle['targ']])
        for pathval in constructCopies(dims, handle['path']):
            handlecopy = handle.copy()
            handlecopy['targets'] = []
            handlecopy['pathvals'] = pathval
            handlecopy['frequency'] = 0
            handlecopy['std_dev'] = 0
            constructTracts(handlecopy, handlecopy, popcopylist)
            for tract in handlecopy['targets']:
                for pop in popcopylist:
                    if tract['target'] == pop['uniquename'] and handlecopy['receptor'] is not None:
                        pop['receptors'][handlecopy['receptor']
                                         ]['MeanExtCon'] = tract['data']['Connectivity']
                        pop['receptors'][handlecopy['receptor']
                                         ]['MeanExtEff'] = tract['data']['MeanEff']
            handles.append(handlecopy)
    return handles


def constructEvents(handleevent, handles, eventlist):
    for handle in handles:
        if handle['name'] == handleevent['hname']:
            valid = True
            for spec, val in zip(handleevent['hpath'], handle['pathvals']):
                if spec != -1 and spec != val:
                    valid = False
            if valid:
                for tract in handle['targets']:
                    if handleevent['etype'] == 'ChangeExtFreq':
                        event = makeEvent(handleevent['time'], 'ChangeExtFreq', handleevent['label'],
                                          tract['target'], handleevent['freq'], tract['data']['TargetReceptor'])
                    if handleevent['etype'] == 'EndTrial':
                        event = makeEvent(handleevent['time'], 'EndTrial', handleevent['label'],
                                          tract['target'], handleevent['freq'])
                    eventlist.append(event)


def printNetData(poppaths, popcopylist, handles):
    for handle in handles:
        print(" ! ", handle['name'], handle['pathvals'])
        for tract in handle['targets']:
            print("  - ", tract['target'])
    for k, x in poppaths.items():
        print(" * ", k)
        for y in popcopylist:
            if y['name'] == k:
                print("  - ", y['uniquename'])
                for targ in y['targets']:
                    print("    - ", targ['target'])


def writeCsv(popcopylist):
    f = open('net.csv', 'w')
    f.write("connection name,from node,to node\n")
    counter = 0
    for pop in popcopylist:
        for tract in pop['targets']:
            # tractname = str(counter) + '_' + tract['name'] + '_' + str(
            # tract['data']['Connectivity']) + '_' +
            # str(tract['data']['MeanEff'])
            tractname = str(counter) + '_' + tract['name']
            f.write(tractname + ',' +
                    pop['uniquename'] + ',' + tract['target'] + "\n")
            counter += 1
    f.flush()
    f.close()


def writeConf(popcopylist):
    f = open('network.conf', 'w')
    for pop in popcopylist:
        f.write('% ' + pop['uniquename'] + '\n')
    for pop in popcopylist:
        f.write('\n\nNeuralPopulation: ' + pop['uniquename'] + '\n')
        f.write('%-------------------------------------------------------\n\n')
        for k, v in pop['data'].items():
            f.write(k + '=' + str(v) + '\n')
        for name, data in pop['receptors'].items():
            f.write('\nReceptor: ' + name + '\n')
            for k, v in data.items():
                f.write(k + '=' + str(v) + '\n')
            f.write('EndReceptor\n')
        for tract in pop['targets']:
            f.write('\nTargetPopulation: ' + tract['target'] + '\n')
            for k, v in tract['data'].items():
                if v != 0:
                    f.write(k + '=' + str(v) + '\n')
            f.write('EndTargetPopulation\n')
        f.write('\nEndNeuralPopulation\n')
    f.flush()
    f.close()


def writePro(eventlist):
    f = open('network.pro', 'w')
    for event in eventlist:
        f.write('\nEventTime ' + str(event['time']) + '\n\n')
        f.write('Type=' + event['etype'] + '\n')
        if event['label'] != '':
            f.write('Label=' + event['label'] + '\n')
        if event['pop'] != '':
            f.write('Population: ' + event['pop'] + '\n')
        if event['receptor'] != '':
            f.write('Receptor: ' + event['receptor'] + '\n')
        if event['pop'] != '':
            f.write('FreqExt=' + str(event['freq']) + '\n')
        f.write('\nEndEvent\n\n')
    f.flush()
    f.close()


def writePickle(trialdata):
    f = open('network.pickle', 'wb')
    pickle.dump(trialdata, f)
    f.flush()
    f.close()



def compileAndRun(trials=1, offset=0, sweepnumber=0, parallel=4):

    if sys.platform == "linux" or sys.platform == "linux2":
        compiler = 'gcc'
    elif sys.platform == "darwin":
        compiler = 'gcc-8'

    c_dir = os.path.join(_package_dir, 'src')

    simfile = os.path.join(getDirectory(sweepnumber), 'sim')
    call('{} -o {} cbgt.c rando2.h -lm -std=c99'.format(compiler, simfile), shell=True, cwd=c_dir)

    seed = np.random.randint(0, 1000)
    for trial in range(0, trials):
        outdir = getDirectory(sweepnumber)
        Popen('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)
    #os.chdir(wkdir)



def compileAndRunSweep(trials=1, offset=0, sweepcount=1, parallel=4):

    if sys.platform == "linux" or sys.platform == "linux2":
        compiler = 'gcc'
        # number of clients for multiprocess
    elif sys.platform == "darwin":
        compiler = 'gcc-8'

    c_dir = os.path.join(_package_dir, 'src')
    # seed = np.random.randint(0, 1000)
    for sweepnumber in range(0, sweepcount):
        simfile = os.path.join(getDirectory(sweepnumber), 'sim')
        call('{} -o {} cbgt.c rando2.h -lm -std=c99'.format(compiler, simfile), shell=True, cwd=c_dir)

    for trial in range(0, trials):
        for sweepnumber in range(0, sweepcount):
            outdir = getDirectory(sweepnumber)
            seed = np.random.randint(0, 1000)
            if (trial * sweepcount + sweepnumber + 1) % parallel == 0:
                call('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)
            else:
                Popen('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)



def compileAndRunSweepALL(trials=1, offset=0, sweepcount=1, parallel=4):

    if sys.platform == "linux" or sys.platform == "linux2":
        compiler = 'gcc'
        # number of clients for multiprocess
    elif sys.platform == "darwin":
        compiler = 'gcc-8'

    c_dir = os.path.join(_package_dir, 'src')

    for sweepnumber in range(0, sweepcount):
        simfile = os.path.join(getDirectory(sweepnumber), 'sim')
        call('{} -o {} cbgt.c rando2.h -lm -std=c99'.format(compiler, simfile), shell=True, cwd=c_dir)

    for sweepnumber in range(0, sweepcount):
        for trial in range(0, trials):
            outdir = getDirectory(sweepnumber)
            seed = np.random.randint(0, 1000)
            if (trial * sweepcount + sweepnumber + 1) % parallel == 0:
                call('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)
            else:
                Popen('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)


def compileAndRunSweepALL_NEW(trials=1, offset=0, sweepcount=1, parallel=4):

    if sys.platform == "linux" or sys.platform == "linux2":
        compiler = 'gcc'
        # number of clients for multiprocess
    elif sys.platform == "darwin":
        compiler = 'gcc-8'

    # c_dir = os.path.join(_package_dir, 'src')

    for sweepnumber in range(0, sweepcount):
        simfile = os.path.join(getDirectory(sweepnumber), 'sim')
        call('{} -o {} cbgt.c rando2.h -lm -std=c99'.format(compiler, simfile), shell=True)

    for sweepnumber in range(0, sweepcount):
        for trial in range(0, trials):
            outdir = getDirectory(sweepnumber)
            seed = np.random.randint(0, 1000)
            if (trial * sweepcount + sweepnumber + 1) % parallel == 0:
                call('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)
            else:
                Popen('./sim -ns -n{} -s{}'.format(str(trial+offset), str(seed+trial+offset)), shell=True, cwd=outdir)



def getCellDefaults():

    return {'N': 250,
            'C': 0.5,
            'Taum': 20,
            'RestPot': -70,
            'ResetPot': -55,
            'Threshold': -50,

            'RestPot_ca': -85,
            'Alpha_ca': 0.5,
            'Tau_ca': 80,
            'Eff_ca': 0.0,

            'tauhm': 20,
            'tauhp': 100,
            'V_h': -60,
            'V_T': 120,
            'g_T': 0}

def getConProb():

    conProb = {'Cx': {'STR': .45,
                        'FSI': 0.45,
                        'Th': .35},
                'D1STR': {'D1STR': .135,
                          'D2STR': .135,
                          'GPi': .57},
                'D2STR': {'D1STR_syn': .15,
                          'D1STR_anti': .135,
                          'D2STR_syn': .135,
                          'GPeP': .74},
                'FSI':  {'FSI': .85,
                        'D1STR': .66,
                        'D2STR': .62},
                'GPeP': {'GPeP': 0.02,
                        'STN': 0.02,
                        'GPi': 1},
                'STN': {'GPeP': 0.0485,
                        'GPi': 1},
                'GPi': {'Th': 0.85},
                'Th': {'STR': 0.45,
                        'FSI': 0.25,
                        'Cx': 0.25,
                        'CxI': 0.25}
                }

    return conProb


def getConEff(**kwargs):

    conEff = {'Cx': {'STR': [0.2, 0.2],
                    'FSI': 0.16,
                    'Th': [0.0335, 0.0335]},
                'D1STR': {'D1STR': .28,
                          'D2STR': .28,
                          'GPi': 1.07},
                'D2STR': {'D1STR_syn': .28,
                          'D1STR_anti': .28,
                          'D2STR_syn': .28,
                          'GPeP': 1.65},
                'FSI':  {'FSI': 1.15,
                        'D1STR': 1.23,
                        'D2STR': 1.23},
                'GPeP': {'GPeP': 1.5,
                        'STN': 0.4,
                        'GPi': 0.012},
                'STN': {'GPeP': [0.07, 4],
                        'GPi': 0.0324},
                'GPi': {'Th': 0.067},
                'Th': {'STR': 0.3,
                        'FSI': 0.3,
                        'Cx': 0.015,
                        'CxI': 0.015}
                }

    kwkeys = list(kwargs)
    nuclei = list(conEff)
    for i in nuclei:
        if i in kwkeys:
            targets = list(kwargs[i])
            for j in targets:
                isList = isinstance(conEff[i][j], list)
                if isList and not isinstance(kwargs[i][j], list):
                    kwargs[i][j] = [kwargs[i][j]]*2
                conEff[i][j] = kwargs[i][j]

    if 'STN' in kwkeys:
        if 'GPeP' in list(kwargs['STN']):
            if not isinstance(kwargs['STN']['GPeP'], list):
                print('STN_GPeP needs AMPA and NMDA efficacies,\nOnly one efficacy found\nfilling with default values')
                conEff['STN']['GPeP'] = [0.07, 4]

    return conEff


def getNetworkDefaults():

    config = {'CxExtEff': 2.8,
              'CxExtFreq': 2.2,

              'STRExtEff': 4.0,
              'STRExtFreq': 1.3,

              'FSIExtEff': 1.55,
              'FSIExtFreq': 3.,

              'STNExtEff': 1.60,
              'STNExtFreq': 4.45,

              'GPiExtEff': 5.9,
              'GPiExtFreq': 0.8,

              'GPeExtEff': 2,
              'GPeExtFreq': 5, #4.85,

              'ThExtEff': 2.5,
              'ThExtFreq': 2.2,

              'BaseStim': [0.00],
              'Stim': [2.6],
              'Dynamic': [30.0],
              'rampingCTX': True}

    return config


def describeBG(**kwargs):

    # makePop(name, receptors=[], data={}, data_overrides={})
    # camP(connections, src, targ, receptor, preset=['all'], connectivity=1,
    #       efficacy=1, STFT=0, STFP=0, STDT=0, STDP=0, name='', cmtype='eff', conmatrix=[])

    config = getNetworkDefaults()
    config.update(kwargs)

    if 'conEff' in list(kwargs):
        conEff = kwargs['conEff']
        # print(conEff)
    else:
        conEff = getConEff()

    if 'conProb' in list(kwargs):
        conProb = kwargs['conProb']
        # print(conProb)
    else:
        conProb = getConProb()

    c = []
    h = []
    cd_pre = getCellDefaults()
    GABA = makeReceptor('GABA', {'Tau': 5, 'RevPot': -70})
    AMPA = makeReceptor('AMPA', {'Tau': 2, 'RevPot': 0})
    NMDA = makeReceptor('NMDA', {'Tau': 100, 'RevPot': 0})

    LIP = makePop("LIP", [GABA,
                        [AMPA, 800, config['CxExtEff'],
                        config['CxExtFreq']], NMDA],
                        cd_pre, {'N': 680})

    camP(c, 'LIP', 'D1STR', ['AMPA', 'NMDA'], ['syn'], conProb['Cx']['STR'], conEff['Cx']['STR'], name='cxd')
    camP(c, 'LIP', 'D2STR',  ['AMPA', 'NMDA'], ['syn'], conProb['Cx']['STR'], conEff['Cx']['STR'], name='cxi')
    camP(c, 'LIP', 'FSI', 'AMPA', ['all'], conProb['Cx']['FSI'], conEff['Cx']['FSI'], name='cxfsi')
    camP(c, 'LIP', 'Th', ['AMPA', 'NMDA'], ['all'], conProb['Cx']['Th'], conEff['Cx']['Th'])


    D1STR = makePop("D1STR", [GABA,
                            [AMPA, 800, config['STRExtEff'],
                            config['STRExtFreq']], NMDA], cd_pre)
    D2STR = makePop("D2STR", [GABA,
                            [AMPA, 800, config['STRExtEff'],
                            config['STRExtFreq']], NMDA], cd_pre)

    camP(c, 'D1STR', 'D1STR', 'GABA', ['syn'], conProb['D1STR']['D1STR'], conEff['D1STR']['D1STR'])
    camP(c, 'D1STR', 'D2STR', 'GABA', ['syn'], conProb['D1STR']['D2STR'], conEff['D1STR']['D2STR'])
    camP(c, 'D1STR', 'GPi', 'GABA', ['syn'], conProb['D1STR']['GPi'], conEff['D1STR']['GPi'], name='direct')

    camP(c, 'D2STR', 'D2STR', 'GABA', ['syn'], conProb['D2STR']['D2STR_syn'], conEff['D2STR']['D2STR_syn'])
    camP(c, 'D2STR', 'D1STR', 'GABA', ['syn'], conProb['D2STR']['D1STR_syn'], conEff['D2STR']['D1STR_syn'])
    camP(c, 'D2STR', 'GPeP', 'GABA', ['syn'], conProb['D2STR']['GPeP'], conEff['D2STR']['GPeP'], name='indirect')

    #######################
    #        TEST         #
    #######################
    # camP(c, 'D1STR', 'D1STR', 'GABA', ['syn'], .1, .28)
    # camP(c, 'D1STR', 'D2STR', 'GABA', ['syn'], .1, .28)
    # camP(c, 'D1STR', 'GPi', 'GABA', ['syn'], .57, 1.07, name='direct')
    # camP(c, 'D2STR', 'D1STR', 'GABA', ['syn'], .135, .28)
    # camP(c, 'D2STR', 'D2STR', 'GABA', ['syn'], .12, .28)

    # camP(c, 'D2STR', 'D1STR', 'GABA', ['anti'], .135, .28)
    # camP(c, 'D2STR', 'D2STR', 'GABA', ['anti'], .12, .28)
    # camP(c, 'D2STR', 'GPeP', 'GABA', ['syn'], .74, 1.65, name='indirect')
    # camP(c, 'D2STR', 'GPeP', 'GABA', ['syn'], .74, 1.65, name='indirect')

    FSI = makePop("FSI", [GABA,
                        [AMPA, 800, config['FSIExtEff'],
                        config['FSIExtFreq']], NMDA],
                        cd_pre, {'C': 0.2, 'Taum': 10})
    camP(c, 'FSI', 'FSI', 'GABA', ['all'], conProb['FSI']['FSI'], conEff['FSI']['FSI'])
    camP(c, 'FSI', 'D1STR', 'GABA', ['all'], conProb['FSI']['D1STR'], conEff['FSI']['D1STR'])
    camP(c, 'FSI', 'D2STR', 'GABA', ['all'], conProb['FSI']['D2STR'], conEff['FSI']['D2STR'])


    GPeP = makePop("GPeP", [[GABA, 2000, 2, 2],
                [AMPA, 800, config['GPeExtEff'],
                config['GPeExtFreq']], NMDA],
                cd_pre, {'N': 2500, 'g_T': 0.06, 'taum':10})
    camP(c, 'GPeP', 'GPeP', 'GABA', ['all'], conProb['GPeP']['GPeP'], conEff['GPeP']['GPeP'])
    camP(c, 'GPeP', 'STNE', 'GABA', ['syn'], conProb['GPeP']['STN'], conEff['GPeP']['STN'])
    camP(c, 'GPeP', 'GPi', 'GABA', ['syn'], conProb['GPeP']['GPi'], conEff['GPeP']['GPi'])


    STNE = makePop("STNE", [GABA,
                            [AMPA, 800, config['STNExtEff'],
                            config['STNExtFreq']], NMDA],
                            cd_pre, {'N': 2500, 'g_T': 0.06})
    camP(c, 'STNE', 'GPeP', ['AMPA', 'NMDA'], ['syn'], conProb['STN']['GPeP'], conEff['STN']['GPeP'])
    camP(c, 'STNE', 'GPi', 'NMDA', ['all'], conProb['STN']['GPi'], conEff['STN']['GPi'])


    GPi = makePop("GPi", [GABA,
                        [AMPA, 800, config['GPiExtEff'],
                        config['GPiExtFreq']], NMDA], cd_pre)
    camP(c, 'GPi', 'Th', 'GABA', ['syn'], conProb['GPi']['Th'], conEff['GPi']['Th'])


    Th = makePop('Th', [GABA,
                        [AMPA, 800, config['ThExtEff'],
                        config['ThExtFreq']], NMDA], cd_pre)
    camP(c, 'Th', 'D1STR', 'AMPA', ['syn'], conProb['Th']['STR'], conEff['Th']['STR'])
    camP(c, 'Th', 'D2STR', 'AMPA', ['syn'], conProb['Th']['STR'], conEff['Th']['STR'])
    camP(c, 'Th', 'FSI', 'AMPA', ['all'], conProb['Th']['FSI'], conEff['Th']['FSI'])
    camP(c, 'Th', 'LIP', 'NMDA', ['all'], conProb['Th']['Cx'], conEff['Th']['Cx'], name='thcx')

    ineuronPops = [FSI]

    if config['rampingCTX']:
        # print('ramping')
        camP(c, 'LIP', 'LIP', ['AMPA', 'NMDA'], ['all'], .13, [0.0127, 0.15])
        camP(c, 'LIP', 'LIPI', ['AMPA', 'NMDA'], ['all'], .0725, [0.013, 0.125])

        LIPI = makePop("LIPI", [GABA, [AMPA, 640, .6, 1.05], NMDA], cd_pre, { 'N': 620, 'C': 0.2, 'Taum': 10})
        camP(c, 'LIPI', 'LIP', 'GABA', ['all'], .5, 1.05)
        camP(c, 'LIPI', 'LIPI', 'GABA', ['all'], 1, 1.075)
        ineuronPops.append(LIPI)

        camP(c, 'Th', 'LIPI', 'NMDA', ['all'], conProb['Th']['CxI'], conEff['Th']['CxI'], name='thcxi')

    action_channel = makeChannel('choices', [GPi, STNE, GPeP, D1STR, D2STR, LIP, Th])

    brain = makeChannel('brain', ineuronPops, [action_channel])

    return (brain, c, h)



def mcInfo(**kwargs):

    config = {'BaseStim': 2.0,
              'WrongStim': 2.50,
              'RightStim': 2.50,
              'Start': 500,
              'Choices': 2,
              'Dynamic': 30}

    config.update(kwargs)

    dims = {'brain': 1, 'choices': config['Choices']}

    hts = []
    hts.append(makeHandle('sensory', 'LIP', ['choices'], 'AMPA', 800, 2.0))
    # hts.append(makeHandle('motor', 'M1', ['choices'], 'AMPA', 800, 2.0))
    # hts.append(makeHandle('cancel', 'STNE', ['choices'], 'AMPA', 800, 1.6))
    hts.append(makeHandle('threshold', 'STNE', ['choices'], 'AMPA', 800, 1.65))
    hts.append(makeHandle('out', 'Th', ['choices']))

    hes = []
    # hes.append(makeHandleEvent('reset', 0, 'sensory', [], config['BaseStim']))
    hes.append(makeHandleEvent('reset', config['Start'], 'sensory', [], config['BaseStim']))
    # hes.append(makeHandleEvent('reset', 0, 'motor', [], config['BaseStim']))
    hes.append(makeHandleEvent('wrong stimulus', config['Start'], 'sensory', [], config['WrongStim']))
    hes.append(makeHandleEvent('right stimulus', config['Start'], 'sensory', [0], config['RightStim']))
    hes.append(makeHandleEvent('hyperdirect', config['Start'], 'threshold', [], config['STNExtFreq']+.75))
    hes.append(makeHandleEvent('hyperdirect', config['Start'], 'threshold', [0], config['STNExtFreq']+.75))
    hes.append(makeHandleEvent('dynamic cutoff', config['Start'], 'out', [], config['Dynamic'], 'EndTrial'))

    houts = []
    houts.append(makeHandleEvent('decision made',
                                 config['Start'], 'out', [], config['Dynamic']))

    # timelimit = 1800
    timelimit = 1200

    return (dims, hts, hes, houts, timelimit)


def ssInfo(**kwargs):

    config = {'BaseStim': 2.0,
              'WrongStim': 2.50,
              'RightStim': 2.54,
              'Start': 500,
              'Choices': 1,
              'Dynamic': 30,
              'CancelDelay': 200,
              'CancelStim': 4.0}

    config.update(kwargs)

    dims = {'brain': 1, 'choices': config['Choices']}

    hts = []
    hts.append(makeHandle('sensory', 'LIP', ['choices'], 'AMPA', 800, 2.0))
    hts.append(makeHandle('motor', 'M1', ['choices'], 'AMPA', 800, 2.0))
    hts.append(makeHandle('cancel', 'STNE', ['choices'], 'AMPA', 800, 1.6))
    hts.append(makeHandle('out', 'Th', ['choices']))

    hes = []
    hes.append(makeHandleEvent('reset', 0, 'sensory', [], config['BaseStim']))
    hes.append(makeHandleEvent('wrong stimulus',
                               config['Start'], 'sensory', [], config['WrongStim']))
    hes.append(makeHandleEvent('right stimulus', config['Start'],
                               'sensory', [0], config['RightStim']))

    hes.append(makeHandleEvent('dynamic cutoff',
                               config['Start'], 'out', [], config['Dynamic'], 'EndTrial'))

    if config['stop'] != 0:
        hes.append(makeHandleEvent('cancel stimulus',
                                   config['Start'] + config['CancelDelay'], 'cancel', [], config['CancelStim']))
        hes.append(makeHandleEvent('cancel input',
                                   config['Start'] + config['CancelDelay'], 'sensory', [], config['BaseStim']))

    houts = []
    houts.append(makeHandleEvent('decision made',
                                 config['Start'], 'out', [], config['Dynamic']))

    timelimit = 1500
    return (dims, hts, hes, houts, timelimit)


def modifyNetwork(popcopylist, connections, **kwargs):
    for key, value in kwargs.items():
        if key == 'popscale':
            for pop in popcopylist:
                pop['data']['N'] *= value
            for path in connections:
                # scale C and then E only if needed
                # maybe some other form of balancing is better
                path['connectivity'] /= value
                if path['connectivity'] > 1:
                    path['efficacy'] *= path['connectivity']
                    path['connectivity'] = 1
        for path in connections:
            if key == path['name'] and not isinstance(value, dict):
                path['efficacy'] *= value


def configureExperiment(**kwargs):
    if 'preset' in kwargs:
        kwargs.update(kwargs['preset'])

    # get network description
    brain, connections, handletypes = describeBG(**kwargs)

    # get description relevant to this experiment and merge
    if kwargs['experiment'] == 'mc':
        dims, hts, handleeventlist, outputevents, timelimit = mcInfo(**kwargs)
    if kwargs['experiment'] == 'ss':
        dims, hts, handleeventlist, outputevents, timelimit = ssInfo(**kwargs)
    for ht in hts:
        handletypes.append(ht)

    # create network populations
    poppaths = constructPopPaths(brain)
    popcopylist = constructPopCopies(dims, brain, poppaths)

    # modify network
    modifyNetwork(popcopylist, connections, **kwargs)

    # create all network connections
    handles = constructHandleCopies(dims, handletypes, poppaths, popcopylist)
    constructConnections(dims, connections, poppaths, popcopylist, **kwargs)

    # create events from handleevents
    eventlist = []
    for he in handleeventlist:
        constructEvents(he, handles, eventlist)
    eventlist.append(makeEvent(timelimit, 'EndTrial'))

    # write files
    # printNetData(poppaths, popcopylist, handles)
    writeCsv(popcopylist)
    writeConf(popcopylist)
    writePro(eventlist)

    trialdata = {'dims': dims,
                 'poppaths': poppaths,
                 'popcopylist': popcopylist,
                 'handles': handles,
                 'eventlist': eventlist,
                 'outputevents': outputevents}
    trialdata.update(kwargs)

    writePickle(trialdata)

    return trialdata


def readTrialResult(sweepnumber, trial):
    directory = getDirectory(sweepnumber)
    f = open(directory + '/popfreqs' + str(trial) + '.dat', "r")
    columns = []
    rawdata = []
    lines = f.readlines()
    for i in range(len(lines)):
        if i == 0:
            columns = lines[i].strip().split("\t")
            for colnum in range(len(columns)):
                rawdata.append([])
        if i > 0:
            data = lines[i].strip().split("\t")
            if(float(data[0]) > 0):
                for colnum, val in zip(range(len(columns)), data):
                    rawdata[colnum].append(val)
    labeled = {}
    for colnum in range(len(columns)):
        labeled[columns[colnum]] = np.array(rawdata[colnum], dtype='float32')

    g = open(directory + '/network.pickle', 'rb')
    trialdata = pickle.load(g)
    trialdata['popfreqs'] = pd.DataFrame(labeled)

    return(trialdata)


def readAllTrialResults(trials, offset=0, sweepcount=1):
    allresults = []
    trialID = lambda f: int(f.split('popfreqs')[1].split('.')[0])

    for sweepnumber in range(sweepcount):
        results = []
        files = os.listdir(getDirectory(sweepnumber))
        trials = [trialID(f) for f in files if 'popfreqs' in f]
        for trial in trials:
            results.append(readTrialResult(sweepnumber, trial + offset))
        allresults.append(results)
    return allresults


def findOutputs(trialdata, df=None):
    if df is None:
        df = trialdata['popfreqs']
    outputs = {}
    for handleevent in trialdata['outputevents']:
        output = {'time': None,
                  'start': handleevent['time'],
                  'delay': None,
                  'pathvals': None,
                  'threshold': handleevent['freq']}
        outputs[handleevent['label']] = output
        for handle in trialdata['handles']:
            if handle['name'] == handleevent['hname']:
                valid = True
                for spec, val in zip(handleevent['hpath'], handle['pathvals']):
                    if spec != -1 and spec != val:
                        valid = False
                if valid:
                    for tract in handle['targets']:
                        for i in range(0, df.shape[0]):
                            curtime = df.at[i, 'Time (ms)']
                            if curtime < output['start']:
                                continue
                            if df.at[i, tract['target']
                                     ] >= output['threshold']:
                                if output['time'] is None or curtime < output['time']:
                                    output['time'] = curtime
                                    output['delay'] = curtime - output['start']
                                    output['pathvals'] = handle['pathvals']
                                break
    trialdata['outputs'] = outputs
    return outputs



def setDirectory(prefix='autotest'):
    global directoryprefix
    directoryprefix = os.path.join(os.path.expanduser('~'), prefix, 'sweeps')


def getDirectory(sweepnumber=0):
    return os.path.join(directoryprefix, str(sweepnumber))


def configureSweep(sc=0, **kwargs):
    config = getNetworkDefaults()
    config.update(kwargs)
    for key, value in kwargs.items():
        if isinstance(value, list):
            selected = {}
            selected.update(kwargs)
            for opt in value:
                selected[key] = opt
                sc = configureSweep(sc, **selected)
            return sc
    configureExperiment(**config)#kwargs)
    directory = getDirectory(sc)
    call('mkdir -p ' + directoryprefix, shell=True)
    call('mkdir -p ' + directory, shell=True)
    for filename in ['network.conf', 'network.pro', 'network.pickle']:
        call('mv ' + filename + ' ' + directory + '/' + filename, shell=True)
    return sc + 1



-----------------------------------
    Population: GPi_0_0
-----------------------------------
  N=100
  C=0.500000 nF
  Membrane time constant=20.000000 ms
  Resting potential=-70.000000 mV
  Reset potential=-55.000000 mV
  Threshold =-50.000000 mV
  RestPot_ca =-85.000000 mV
  Alpha_ca =0.500000 mV
  Tau_ca =80.000000 mV
  Eff_ca =0.000000 mV
  tauhm=20.000000 ms
  tauhp=100.000000 ms
  V_h=-60.000000 mV
  V_T=120.000000 mV
  g_T=0.000000 mS
Receptor 0: GABA (Mg block: 0)
  Tau=5.000000 ms
  Reversal potential=-70.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Receptor 1: AMPA (Mg block: 0)
  Tau=2.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=0.800000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=5.900000 nS
  Ext connections=800.000000
Receptor 2: NMDA (Mg block: 1)
  Tau=100.000000 ms
  Reversal potential=0.000000 mV
  Ext frequency=0.000000 Hz
  Ext frequency SD=0.000000 Hz
  Ext efficacy=0.000000 nS
  Ext connections=0.000000
Synapses targeting population: Th_0_0 (14)
   Target receptor code=0
  Connectivity=1.000000
  Mean efficacy=0.142375
EndPopulation



#!usr/bin/env python
import os
import glob

modules = glob.glob(os.path.dirname(__file__)+"/*.py")
__all__ = [ os.path.basename(f)[:-3] for f in modules]
_package_dir = os.path.dirname(os.path.realpath(__file__))


__version__ = '0.0.1'

from . import netgen
from . import analyzefx
from . import sim
from . import vis

try:
    import cbgt
except ImportError:
    pass



from setuptools import setup, find_packages
from setuptools import Extension
import numpy as np

ext_modules = [Extension('cbgt', ['src/cbgt.c'])]
package_data = {'cbgt': ['src/*.c', 'src/*.h', 'docs/*.md',
                'docs/*.txt', 'params/*.py', 'params/*.tex']}
cbgt_packages = ['cbgt', 'cbgt.src']

setup(
    name='CBGT',
    version='0.0.1',
    author='Kyle Dunovan, Catalina Vich, Matthew Clapp, Timothy Verstynen, Jonathan Rubin, Wei Wei, and Xiao Jing Wang',
    author_email='dunovank@gmail.com',
    url='http://github.com/CoAxLab/bgNetwork',
    packages=cbgt_packages,
    package_data=package_data,
    description='CBGT contains code for implementing the CBGT network and drift-diffusion model (DDM) fits described in the manuscript Reward-driven changes in striatal pathway competition shape evidence evaluation in decision-making.',
    install_requires=['numpy==1.11.3', 'pandas>=0.15.1', 'matplotlib>=1.4.3', 'seaborn>=0.5.1', 'future'],
    setup_requires=['numpy==1.11.3', 'pandas>=0.15.1', 'matplotlib>=1.4.3', 'seaborn>=0.5.1', 'future'],
    include_dirs = [np.get_include()],
    classifiers=['Environment :: Console',
                'License :: OSI Approved :: BSD License',
                'Intended Audience :: Science/Research',
                'Development Status :: 3 - Alpha',
                'Programming Language :: Python',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.6',
                'Topic :: Scientific/Engineering',
                 ],
    ext_modules = ext_modules
)



import os
import bgNetwork.netgen as ng
import numpy as np
import pandas as pd

roundRT = lambda x, ndec: int(round(float(str(x)[:5]), ndec)*1000)
zscore_series = lambda s: (s - s.mean()) / s.std()
normalize_series = lambda s: (s - s.min()) / (s.max() - s.min())

def print_results_summary(behavdf):
    cond = behavdf.cond.iloc[0]
    pLeft = behavdf.acc.mean()
    corRT = behavdf[behavdf.acc==1].rtms.mean()
    corMedRT = behavdf[behavdf.acc==1].rtms.median()
    corRTvar = behavdf[behavdf.acc==1].rtms.sem()*2
    errRT = 999.99; errRTvar = 0.0
    if behavdf[behavdf.acc==0].shape[0]>1:
        errRT = behavdf[behavdf.acc==0].rtms.mean()
        errRTvar = behavdf[behavdf.acc==0].rtms.sem()*2
    resultsSummary = 'Cond: {}\n\tMean CorRT:\t{:.2f} ({:.2f})\n\tMed CorRT:\t{:.2f}\n\tAvg ErrRT:\t{:.2f} ({:.2f})\n\tP(Left):\t{:.4f}'
    print(resultsSummary.format(cond.upper(),corRT, corRTvar, corMedRT, errRT, errRTvar, pLeft))

def print_trial_acc_rt(results, t=0):
    bdata = ng.findOutputs(results[t])['decision made']
    rt = bdata['time']
    if bdata['pathvals'] is not None:
        acc = ['Error' if bdata['pathvals'][0] else 'Correct'][0]
        print('RT:\t{:.2f}ms\nAcc:\t{}\n'.format(rt,acc))
        return 1
    else:
        return 0

def get_msn_rates(sdf):
    dL = sdf[sdf['population']=='dMSN0'].reset_index(drop=True)
    dR = sdf[sdf['population']=='dMSN1'].reset_index(drop=True)
    iL = sdf[sdf['population']=='iMSN0'].reset_index(drop=True)
    iR = sdf[sdf['population']=='iMSN1'].reset_index(drop=True)
    dlx = dL.fillna(method='pad', axis=1).convert_objects(convert_numeric=True)
    drx = dR.fillna(method='pad', axis=1).convert_objects(convert_numeric=True)
    ilx = iL.fillna(method='pad', axis=1).convert_objects(convert_numeric=True)
    irx = iR.fillna(method='pad', axis=1).convert_objects(convert_numeric=True)
    return [dlx, drx, ilx, irx]

def get_avgMSN_traces(msnDFs, window=None):

    #rolltime = lambda df, w: df.rolling(w, axis=1).mean()
    rolltime = lambda df, w: df.ix[:, 200:].rolling(w, axis=1).mean().iloc[:, w:]
    getMean = lambda df: df.groupby('cond2').mean().ix[:, 205:].values
    getErr = lambda df: df.groupby('cond2').std().ix[:, 205:].values / 1.4
    if window is not None:
        for i, msndf in enumerate(msnDFs):
            msndf.ix[:, 200+window:] = rolltime(msndf, window)
            msnDFs[i] = msndf
            #msnDFs = [rolltime(msnPop, window) for msnPop in msnDFs]
    ys = [getMean(msnPop) for msnPop in msnDFs]
    ysErr = [getErr(msnPop) for msnPop in msnDFs]
    return ys, ysErr

def get_single_trial_ratedf(spikedf, trial=0):
    spikedf = spikedf.drop(columns=['idx', 'cond', 'cond2'])
    sdf = spikedf[spikedf.trial==trial].T.reset_index(drop=True)
    sdf.columns = sdf.iloc[0, :].values
    sdf = sdf.drop(0)
    sdf = sdf.fillna(np.nan)
    return sdf

def get_mean_firing_rates(spikedf, window=None):
    avgdf = spikedf.groupby(['population']).mean().iloc[:, 1:].T.reset_index(drop=True)
    return avgdf

def get_firing_rates(results, window=None, getavg=False, cond='test', idx=1):
    dflist = []
    condABC = {'low': 'a', 'med': 'b', 'high': 'c', 'test':'x'}
    for i, res in enumerate(results):
        ratedf = res['popfreqs']
        if window is not None:
            ratedf = ratedf.rolling(window).mean()
        ratedf = ratedf.T
        if ratedf.shape[1]==0:
            break
        ratedf.insert(0, 'trial', i)
        dflist.append(ratedf)
    sdf = pd.concat(dflist).reset_index()
    sdf = sdf.rename(columns={'index':'population'})
    sdf = rename_populations(sdf)
    sdf.insert(1, 'cond2', condABC[cond])
    sdf.insert(1, 'cond', cond)
    sdf.insert(1, 'idx', idx)
    sdf['trial'] = sdf.trial.values #+ 1

    if getavg:
        return get_mean_firing_rates(sdf)
    return sdf

def analyze_network_behavior(results, preset, stimArray, cond='test', outdir=None, idx=1, conEff=None):
    rtms, choice, acc, decisions = [], [], [], []
    condABC = {'low': 'a', 'med': 'b', 'high': 'c'}
    condLevel = {'low': 1, 'med': 2, 'high': 3, 'test': 0}
    for result in results:
        decisions.append(ng.findOutputs(result)['decision made'])
    for decision in decisions:
        if decision['pathvals'] is None:
            choice.append(np.nan)
            acc.append(np.nan)
            rtms.append(np.nan)
            continue
        elif decision['pathvals'] == [0]:
            choice.append(0)
            acc.append(1)
        else:
            choice.append(1)
            acc.append(0)
        rtms.append(decision['delay'])
    behav_arr = [np.asarray(arr) for arr in [choice, acc, rtms]]
    behavdf = pd.DataFrame(dict(zip(['choice', 'acc', 'rtms'], behav_arr)))
    behavdf['rtms'] = behavdf.rtms.values + 200
    behavdf['rt'] = behavdf.rtms.values * .001
    behavdf.insert(0, 'trial', np.arange(behavdf.shape[0]))
    behavdf.insert(0, 'response', behavdf.acc.values)
    behavdf.insert(0, 'cond', cond)
    behavdf.insert(0, 'stim', stimArray[:behavdf.shape[0]])
    behavdf.insert(0, 'subj_idx', idx)
    behavdf['cond2'] = condABC[cond]
    behavdf['level'] = condLevel[cond]

    dL,dR = preset[0]['cxd']['mult']
    iL,iR = preset[0]['cxi']['mult']

    idx_CxSTR_w = 1
    if conEff is not None:
        idx_CxSTR_w = conEff['Cx']['STR'][0]

    behavdf['wdL'] = dL * idx_CxSTR_w
    behavdf['wdR'] = dR * idx_CxSTR_w
    behavdf['wiL'] = iL * idx_CxSTR_w
    behavdf['wiR'] = iR * idx_CxSTR_w

    behavdf = behavdf[['subj_idx', 'cond', 'cond2', 'level', 'trial', 'stim', 'wdL', 'wiL', 'wdR', 'wiR', 'response', 'acc', 'choice', 'rtms', 'rt']]
    print_results_summary(behavdf)
    if outdir is not None:
        behavdfName = os.path.join(outdir,'cbgt_behavdf_{}_idx{}.csv'.format(cond, idx))
        outdir = os.path.join(outdir, behavdfName)
        behavdf.to_csv(behavdfName, index=False)
    return behavdf


def make_wts_df(presets=None, conds=['low', 'med', 'high'], ntrials=1000):

    condlist, dleft, dright, ileft, iright = [], [], [], [], []
    for i, pre in enumerate(presets):
        dL,dR = pre[0]['cxd']['mult']
        iL,iR = pre[0]['cxi']['mult']
        condlist.extend([conds[i]]*ntrials)
        dleft.extend([dL]*ntrials)
        dright.extend([dR]*ntrials)
        ileft.extend([iL]*ntrials)
        iright.extend([iR]*ntrials)
    weightdf = pd.DataFrame({'cond':condlist, 'wdL': dleft, 'wiL':ileft, 'wdR':dright, 'wiR':iright})[['cond', 'wdL', 'wiL', 'wdR', 'wiR']]
    return weightdf

def rename_populations(sdf):
    popNames = []
    oldPopNames = sdf.population.unique().tolist()
    for pop in oldPopNames:
        poplist = pop.split('_')
        if len(poplist)==2:
            popName = poplist[0]
        else:
            popName = ''.join(poplist[:2])
        if 'D1STR' in popName:
            popName = 'dMSN'+popName[-1]
        elif 'D2STR' in popName:
            popName = 'iMSN'+popName[-1]
        elif 'STNE' in popName:
            popName = 'STN'+popName[-1]
        elif 'GPeP' in popName:
            popName = 'GPe'+popName[-1]
        elif 'Time (ms)' in popName:
            popName = 'time'
        popNames.append(popName)

    sdfNew = sdf.replace({'population': dict(zip(oldPopNames, popNames))})
    return sdfNew


def get_cbgt_covariates(df, msndf, conds=['low', 'med', 'high']):

    if np.any(msndf.isna()):
        msnPad = msndf.fillna(method='pad', axis=1).convert_objects(convert_numeric=True)
    else:
        msnPad = msndf.copy()

    c_list = []
    for c in conds:
        msn_c = msnPad[msnPad.cond==c]

        idx_list = []
        for idx in df.subj_idx.unique():
            idf = df[(df.subj_idx==idx)&(df.cond==c)&(~df.rt.isna())]
            trials = idf.trial.values
            msn_ci = msn_c[(msn_c.idx==idx)&(msn_c.trial.isin(trials))]
            msn_ci = msn_ci.drop_duplicates()

            DL = msn_ci[msn_ci.population=='dMSN0'].iloc[:, 205:].values
            DR = msn_ci[msn_ci.population=='dMSN1'].iloc[:, 205:].values
            IL = msn_ci[msn_ci.population=='iMSN0'].iloc[:, 205:].values
            IR = msn_ci[msn_ci.population=='iMSN1'].iloc[:, 205:].values

            idf['dL_rate'] = DL.sum(axis=1)
            idf['dR_rate'] = DR.sum(axis=1)
            idf['iL_rate'] = IL.sum(axis=1)
            idf['iR_rate'] = IR.sum(axis=1)

            idf['D_lrdiff_sum'] = np.sum(DL - DR, axis=1)
            idf['DI_ldiff_sum'] = np.sum(DL - IL, axis=1)
            idf['I_lrdiff_sum'] = np.sum(IL - IR, axis=1)
            idf['I_LR_mean'] = (idf.iL_rate + idf.iR_rate)/2

            idx_list.append(idf)

        c_list.append(pd.concat(idx_list).reset_index(drop=True))

    df_new = pd.concat(c_list).reset_index(drop=True)
    return df_new


def norm_covariates(bdf):
    zscore_series = lambda s: (s - s.mean()) / s.std()
    normalize_series = lambda s: (s - s.min()) / (s.max() - s.min())
    xdf = bdf.copy() #fillna(bdf.mean())

    xdf['ndL_rate'] = normalize_series(xdf.dL_rate.values)
    xdf['niL_rate'] = normalize_series(xdf.iL_rate.values)
    xdf['ndR_rate'] = normalize_series(xdf.dR_rate.values)
    xdf['niR_rate'] = normalize_series(xdf.iR_rate.values)

    xdf['nD_lrdiff_sum'] = normalize_series(xdf.D_lrdiff_sum.values)
    xdf['nDI_ldiff_sum'] = normalize_series(xdf.DI_ldiff_sum.values)
    xdf['nI_lrdiff_sum'] = normalize_series(xdf.I_lrdiff_sum.values)
    xdf['nI_LR_mean'] = normalize_series(xdf.I_LR_mean.values)
    #xdf['I_LR_mean2'] = (xdf.iL_rate + xdf.iR_rate)/2
    #xdf['nI_LR_mean2'] = normalize_series(xdf['I_LR_mean2'])
    xdf = xdf.reset_index(drop=True)
    return xdf



##########################################################
#                       OLD                              #
##########################################################


#
# def get_cbgt_covariates(df, msnDFs):
#
#     conds = df.cond.unique().tolist()
#
#     dlx, drx, ilx, irx = msnDFs
#     dL = dlx.reset_index(drop=True)
#     iL = ilx.reset_index(drop=True)
#     dR = drx.reset_index(drop=True)
#     iR = irx.reset_index(drop=True)
#
#     for c in conds:
#
#         trials = df[(df.cond==c)&(~df.rt.isna())].trial.values
#         rt = df[(df.cond==c)&(~df.rt.isna())]['rtms'].astype(int).reset_index(drop=True).values
#         acc = df[(df.cond==c)&(~df.rt.isna())]['acc'].astype(int).reset_index(drop=True).values
#
#         dL_rate_list = []; iL_rate_list = [];
#         dR_rate_list = []; iR_rate_list = [];
#
#         d_lrdiff_sum_list = []
#         di_ldiff_sum_list = []
#         i_lrdiff_sum_list = []
#         i_lrmean_sum_list = []
#
#         for i, t in enumerate(trials):
#             DLFull = np.squeeze(dL[(dL.cond==c)&(dL.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             ILFull = np.squeeze(iL[(iL.cond==c)&(iL.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             DRFull = np.squeeze(dR[(dR.cond==c)&(dR.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             IRFull = np.squeeze(iR[(iR.cond==c)&(iR.trial==t)].ix[:, 205:].dropna(axis=1).values)
#
#             dL_rate_list.append(np.sum(DLFull))
#             iL_rate_list.append(np.sum(ILFull))
#             dR_rate_list.append(np.sum(DRFull))
#             iR_rate_list.append(np.sum(IRFull))
#
#             d_lrdiff_sum_list.append(np.sum(DLFull - DRFull))
#             di_ldiff_sum_list.append(np.sum(DLFull - ILFull))
#             i_lrdiff_sum_list.append(np.sum(ILFull - IRFull))
#             i_lrmean_sum_list.append(np.column_stack([ILFull, IRFull]).mean(1).sum())
#
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'dL_rate'] = dL_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'iL_rate'] = iL_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'dR_rate'] = dR_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'iR_rate'] = iR_rate_list
#
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'D_lrdiff_sum'] = d_lrdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'DI_ldiff_sum'] = di_ldiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_lrdiff_sum'] = i_lrdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_LR_mean'] = i_lrmean_sum_list
#         # df.ix[(df.cond==c)&(~df.rt.isna()), 'I_LR_mean2'] = (df.ix[(df.cond==c)&(~df.rt.isna())].iL_rate + df.ix[(df.cond==c)&(~df.rt.isna())].iR_rate)/2
#
#     return df


# def get_cbgt_covariates(df, msnDFs):
#
#     conds = df.cond.unique().tolist()
#
#     dlx, drx, ilx, irx = msnDFs
#     dL = dlx.reset_index(drop=True)
#     iL = ilx.reset_index(drop=True)
#     dR = drx.reset_index(drop=True)
#     iR = irx.reset_index(drop=True)
#
#     for c in conds:
#
#         trials = df[(df.cond==c)&(~df.rt.isna())].trial.values
#         rt = df[(df.cond==c)&(~df.rt.isna())]['rtms'].astype(int).reset_index(drop=True).values
#         acc = df[(df.cond==c)&(~df.rt.isna())]['acc'].astype(int).reset_index(drop=True).values
#
#         d_diff_list, di_diff_list, id_diff_list, imax_list = [], [], [], []
#         d_diff_sum_list = []; d_lrdiff_sum_list = []; di_ldiff_sum_list = []
#         di_lrdiff_sum_list = []; di_rdiff_sum_list = []; di_lrdiff_sum_list = []
#         i_lrmean_sum_list = []; id_rldiff_sum_list = []; i_rldiff_sum_list = []
#         dL_rate_list = []; iL_rate_list = []; dR_rate_list = []; iR_rate_list = []; i_lrdiff_sum_list = []
#
#         for i, t in enumerate(trials):
#             DLFull = np.squeeze(dL[(dL.cond==c)&(dL.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             ILFull = np.squeeze(iL[(iL.cond==c)&(iL.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             DRFull = np.squeeze(dR[(dR.cond==c)&(dR.trial==t)].ix[:, 205:].dropna(axis=1).values)
#             IRFull = np.squeeze(iR[(iR.cond==c)&(iR.trial==t)].ix[:, 205:].dropna(axis=1).values)
#
#             dL_rate_list.append(np.sum(DLFull))
#             iL_rate_list.append(np.sum(ILFull))
#             dR_rate_list.append(np.sum(DRFull))
#             iR_rate_list.append(np.sum(IRFull))
#
#             d_lrdiff_sum_list.append(np.sum(DLFull - DRFull))
#             di_ldiff_sum_list.append(np.sum(DLFull - ILFull))
#             i_lrdiff_sum_list.append(np.sum(ILFull - IRFull))
#             i_lrmean_sum_list.append(np.column_stack([ILFull, IRFull]).mean(1).sum())
#             i_lrmean_sum_list2.append(np.column_stack([ILFull, IRFull]).mean(1).sum())
#
#             di_rdiff_sum_list.append(np.sum(DRFull - IRFull))
#             di_lrdiff_sum_list.append(np.sum(DLFull - ILFull) - np.sum(DRFull - IRFull))
#             id_rldiff_sum_list.append(np.sum(IRFull - DRFull) - np.sum(ILFull - DLFull))
#             i_rldiff_sum_list.append(np.sum(IRFull - ILFull))
#
#             if acc[i]:
#                 id_diff_list.append(np.sum(ILFull - DLFull))
#                 di_diff_list.append(np.sum(DLFull - ILFull))
#                 d_diff_sum_list.append(np.sum(DLFull - DRFull))
#                 imax_list.append(np.sum(ILFull))
#             else:
#                 d_diff_sum_list.append(np.sum(DRFull - DLFull))
#                 id_diff_list.append(np.sum(IRFull - DRFull))
#                 di_diff_list.append(np.sum(DRFull - IRFull))
#                 imax_list.append(np.sum(IRFull))
#
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'dL_rate'] = dL_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'iL_rate'] = iL_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'dR_rate'] = dR_rate_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'iR_rate'] = iR_rate_list
#
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'D_lrdiff_sum'] = d_lrdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'DI_ldiff_sum'] = di_ldiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_lrdiff_sum'] = i_lrdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_LR_mean'] = i_lrmean_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'DI_rdiff_sum'] = di_rdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'DI_lrdiff_sum'] = di_lrdiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'ID_diff_chosen'] = id_diff_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'DI_diff_chosen'] = di_diff_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'D_diff_sum'] = d_diff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_sum_chosen'] = imax_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_LR_mean'] = (df.iL_rate + df.iR_rate)/2
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_avgLR_sum'] = i_lrmean_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'ID_rldiff_sum'] = id_rldiff_sum_list
#         df.ix[(df.cond==c)&(~df.rt.isna()), 'I_rldiff_sum'] = i_rldiff_sum_list
#     return df
#
#
# def norm_covariates(bdf):
#     zscore_series = lambda s: (s - s.mean()) / s.std()
#     normalize_series = lambda s: (s - s.min()) / (s.max() - s.min())
#     xdf = bdf.copy() #fillna(bdf.mean())
#
#     xdf['ndL_rate'] = normalize_series(xdf.dL_rate.values)
#     xdf['niL_rate'] = normalize_series(xdf.iL_rate.values)
#     xdf['ndR_rate'] = normalize_series(xdf.dR_rate.values)
#     xdf['niR_rate'] = normalize_series(xdf.iR_rate.values)
#
#     xdf['nD_lrdiff_sum'] = normalize_series(xdf.D_lrdiff_sum.values)
#     xdf['nDI_ldiff_sum'] = normalize_series(xdf.DI_ldiff_sum.values)
#     xdf['nI_lrdiff_sum'] = normalize_series(xdf.I_lrdiff_sum.values)
#     xdf['nI_LR_mean'] = normalize_series(xdf.I_LR_mean.values)
#
#     xdf['zD_diff_sum'] = zscore_series(xdf.D_diff_sum.values)
#     xdf['zD_lrdiff_sum'] = zscore_series(xdf.D_lrdiff_sum.values)
#     xdf['nD_diff_sum'] = normalize_series(xdf.D_diff_sum.values)
#     xdf['nDI_diff_chosen'] = normalize_series(xdf.DI_ldiff_sum.values)
#
#     xdf['nDI_rdiff_sum'] = normalize_series(xdf.DI_rdiff_sum.values)
#     xdf['nDI_lrdiff_sum'] = normalize_series(xdf.DI_lrdiff_sum.values)
#     xdf['nI_sum_chosen'] = normalize_series(xdf.I_sum_chosen.values)
#     xdf['nID_diff_chosen'] = normalize_series(xdf.ID_diff_chosen.values)
#     xdf['nI_avgLR_sum'] = normalize_series(xdf.I_avgLR_sum.values)
#     xdf['nID_rldiff_sum'] = normalize_series(xdf.ID_rldiff_sum.values)
#     xdf['nI_rldiff_sum'] = normalize_series(xdf.I_rldiff_sum.values)
#
#     for dff in [xdf]:
#         dff['IL_DR_diff'] = dff.iL_rate - dff.dR_rate
#         dff['nIL_DR_diff'] = normalize_series(dff.IL_DR_diff)
#         dff['iLdR_diff'] = dff.iL_rate - dff.dR_rate
#         dff['niLdR_diff'] = normalize_series(dff.iLdR_diff)
#         dff['dLiR_avg'] = (dff.dL_rate + dff.iR_rate)/2
#         dff['dLiR_avg'] = normalize_series(dff.dLiR_avg)
#         dff['I_lrdiff_sum'] = dff.iL_rate - dff.iR_rate
#         dff['nI_lrdiff_sum'] = normalize_series(dff['I_lrdiff_sum'])
#         dff['ID_lrdiff_sum'] = (dff.iL_rate - dff.dL_rate) - (dff.iR_rate - dff.dR_rate)
#         dff['nID_lrdiff_sum'] = normalize_series(dff['ID_lrdiff_sum'])
#         #dff['I_LR_mean'] = (dff.iL_rate + dff.iR_rate)/2
#         #dff['nI_LR_mean'] = normalize_series(dff['I_LR_mean'])
#         dff['di_ldiff_sum'] = dff.dL_rate - dff.iL_rate
#         dff['ndi_ldiff_sum'] = normalize_series(dff.dL_rate - dff.iL_rate)
#
#     xdf = xdf.reset_index(drop=True)
#     return xdf
